@@ -33,6 +33,9 @@
 #  ifdef HAVE_NETINET_TCP_H
 #   include <netinet/tcp.h>
 #  endif /* HAVE_NETINET_TCP_H */
+#  ifdef HAVE_SIGNAL_H
+#   include <signal.h>
+#  endif /* HAVE_SIGNAL_H */
 #endif /* WIN32 */
 
 #include "ctx.h"
@@ -961,6 +964,7 @@ grn_com_sopen(grn_ctx *ctx, grn_com_event *ev,
   grn_com *cs = NULL;
   int getaddrinfo_result;
   struct addrinfo *bind_address_info = NULL;
+  struct addrinfo hints;
   char port_string[6]; /* ceil(log10(65535)) + 1 ('\0')*/
 
   GRN_API_ENTER;
@@ -968,15 +972,21 @@ grn_com_sopen(grn_ctx *ctx, grn_com_event *ev,
     bind_address = "0.0.0.0";
   }
   snprintf(port_string, sizeof(port_string), "%d", port);
+  memset(&hints, 0, sizeof(struct addrinfo));
+  hints.ai_family = PF_UNSPEC;
+  hints.ai_socktype = SOCK_STREAM;
+  hints.ai_flags = AI_NUMERICSERV;
   getaddrinfo_result = getaddrinfo(bind_address, port_string,
-                                   NULL, &bind_address_info);
+                                   &hints, &bind_address_info);
   if (getaddrinfo_result != 0) {
     switch (getaddrinfo_result) {
+#ifdef EAI_MEMORY
     case EAI_MEMORY:
       ERR(GRN_NO_MEMORY_AVAILABLE,
           "getaddrinfo: <%s:%s>: %s",
           bind_address, port_string, gai_strerror(getaddrinfo_result));
       break;
+#endif
 #ifdef EAI_SYSTEM
     case EAI_SYSTEM:
       SERR("getaddrinfo");
