@@ -15,14 +15,13 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-#include <string.h>
-#include <stdlib.h>
 #include "geo.h"
-#include "ii.h"
-#include "db.h"
 #include "pat.h"
 #include "util.h"
 
+#include <string.h>
+#include <stdlib.h>
+
 typedef struct {
   grn_id id;
   double d;
@@ -34,6 +33,22 @@ typedef struct
   int key_size;
 } mesh_entry;
 
+typedef struct {
+  grn_obj *pat;
+  grn_obj top_left_point_buffer;
+  grn_obj bottom_right_point_buffer;
+  grn_geo_point *top_left;
+  grn_geo_point *bottom_right;
+  grn_geo_point base;
+  grn_geo_point min;
+  grn_geo_point max;
+  int start;
+  int end;
+  int distance;
+  int diff_bit;
+  grn_geo_mesh_direction direction;
+} in_rectangle_data;
+
 static int
 compute_diff_bit(uint8_t *geo_key1, uint8_t *geo_key2)
 {
@@ -693,11 +708,6 @@ exit :
   return ctx->rc;
 }
 
-typedef enum {
-  MESH_LATITUDE,
-  MESH_LONGITUDE
-} mesh_direction;
-
 grn_rc
 grn_selector_geo_in_rectangle(grn_ctx *ctx, grn_obj *obj,
                               grn_obj **args, int nargs,
@@ -718,17 +728,22 @@ grn_selector_geo_in_rectangle(grn_ctx *ctx, grn_obj *obj,
   return ctx->rc;
 }
 
-grn_rc
-grn_geo_select_in_rectangle(grn_ctx *ctx, grn_obj *index,
-                            grn_obj *top_left_point, grn_obj *bottom_right_point,
-                            grn_obj *res, grn_operator op)
+static grn_rc
+in_rectangle_data_prepare(grn_ctx *ctx, grn_obj *index,
+                          grn_obj *top_left_point,
+                          grn_obj *bottom_right_point,
+                          const char *process_name,
+                          in_rectangle_data *data)
 {
   grn_id domain;
-  grn_obj *pat, top_left_point_, bottom_right_point_;
-  grn_geo_point *top_left, *bottom_right;
 
-  pat = grn_ctx_at(ctx, index->header.domain);
-  domain = pat->header.domain;
+  if (!index) {
+    ERR(GRN_INVALID_ARGUMENT, "%s: index column is missing", process_name);
+    goto exit;
+  }
+
+  data->pat = grn_ctx_at(ctx, index->header.domain);
+  domain = data->pat->header.domain;
   if (domain != GRN_DB_TOKYO_GEO_POINT && domain != GRN_DB_WGS84_GEO_POINT) {
     char name[GRN_TABLE_MAX_KEY_SIZE];
     int name_size = 0;
@@ -742,45 +757,50 @@ grn_geo_select_in_rectangle(grn_ctx *ctx, grn_obj *index,
       name_size = strlen(name);
     }
     ERR(GRN_INVALID_ARGUMENT,
-        "geo_in_rectangle(): index table must be "
+        "%s: index table must be "
         "TokyoGeoPoint or WGS84GeoPoint key type table: <%.*s>",
-        name_size, name);
+        process_name, name_size, name);
     goto exit;
   }
 
   if (top_left_point->header.domain != domain) {
-    GRN_OBJ_INIT(&top_left_point_, GRN_BULK, 0, domain);
-    if (grn_obj_cast(ctx, top_left_point, &top_left_point_, 0)) { goto exit; }
-    top_left_point = &top_left_point_;
-  }
-  top_left = GRN_GEO_POINT_VALUE_RAW(top_left_point);
-  if (bottom_right_point->header.domain != domain) {
-    GRN_OBJ_INIT(&bottom_right_point_, GRN_BULK, 0, domain);
-    if (grn_obj_cast(ctx, bottom_right_point, &bottom_right_point_, 0)) {
+    grn_obj_reinit(ctx, &(data->top_left_point_buffer), domain, GRN_BULK);
+    if (grn_obj_cast(ctx, top_left_point, &(data->top_left_point_buffer),
+                     GRN_FALSE)) {
       goto exit;
     }
-    bottom_right_point = &bottom_right_point_;
+    top_left_point = &(data->top_left_point_buffer);
   }
-  bottom_right = GRN_GEO_POINT_VALUE_RAW(bottom_right_point);
+  data->top_left = GRN_GEO_POINT_VALUE_RAW(top_left_point);
+
+  if (bottom_right_point->header.domain != domain) {
+    grn_obj_reinit(ctx, &(data->bottom_right_point_buffer), domain, GRN_BULK);
+    if (grn_obj_cast(ctx, bottom_right_point, &(data->bottom_right_point_buffer),
+                     GRN_FALSE)) {
+      goto exit;
+    }
+    bottom_right_point = &(data->bottom_right_point_buffer);
+  }
+  data->bottom_right = GRN_GEO_POINT_VALUE_RAW(bottom_right_point);
+
   {
-    mesh_direction direction;
-    int distance, latitude_distance, longitude_distance;
-    int i, start, end, diff_bit;
-    grn_geo_point *geo_point_input, geo_point_base, geo_point_min, geo_point_max;
-    uint8_t geo_key_input[sizeof(grn_geo_point)];
-    uint8_t geo_key_base[sizeof(grn_geo_point)];
+    grn_geo_point *top_left, *bottom_right;
+
+    top_left = data->top_left;
+    bottom_right = data->bottom_right;
 
     if (top_left->latitude < 0 || top_left->longitude < 0 ||
         bottom_right->latitude < 0 || bottom_right->longitude < 0) {
       ERR(GRN_FUNCTION_NOT_IMPLEMENTED,
-          "geo_in_rectangle(): negative coordinate is not implemented.");
+          "%s: negative coordinate is not implemented.", process_name);
       goto exit;
     }
 
     if (top_left->latitude >= GRN_GEO_MAX_LATITUDE) {
       ERR(GRN_INVALID_ARGUMENT,
-          "geo_in_rectangle(): top left point's latitude is too big: "
+          "%s: top left point's latitude is too big: "
           "<%d>(max:%d): (%d,%d) (%d,%d)",
+          process_name,
           GRN_GEO_MAX_LATITUDE, top_left->latitude,
           top_left->latitude, top_left->longitude,
           bottom_right->latitude, bottom_right->longitude);
@@ -789,8 +809,9 @@ grn_geo_select_in_rectangle(grn_ctx *ctx, grn_obj *index,
 
     if (top_left->longitude >= GRN_GEO_MAX_LONGITUDE) {
       ERR(GRN_INVALID_ARGUMENT,
-          "geo_in_rectangle(): top left point's longitude is too big: "
+          "%s: top left point's longitude is too big: "
           "<%d>(max:%d): (%d,%d) (%d,%d)",
+          process_name,
           GRN_GEO_MAX_LONGITUDE, top_left->longitude,
           top_left->latitude, top_left->longitude,
           bottom_right->latitude, bottom_right->longitude);
@@ -799,8 +820,9 @@ grn_geo_select_in_rectangle(grn_ctx *ctx, grn_obj *index,
 
     if (bottom_right->latitude >= GRN_GEO_MAX_LATITUDE) {
       ERR(GRN_INVALID_ARGUMENT,
-          "geo_in_rectangle(): bottom right point's latitude is too big: "
+          "%s: bottom right point's latitude is too big: "
           "<%d>(max:%d): (%d,%d) (%d,%d)",
+          process_name,
           GRN_GEO_MAX_LATITUDE, bottom_right->latitude,
           top_left->latitude, top_left->longitude,
           bottom_right->latitude, bottom_right->longitude);
@@ -809,101 +831,428 @@ grn_geo_select_in_rectangle(grn_ctx *ctx, grn_obj *index,
 
     if (bottom_right->longitude >= GRN_GEO_MAX_LONGITUDE) {
       ERR(GRN_INVALID_ARGUMENT,
-          "geo_in_rectangle(): bottom right point's longitude is too big: "
+          "%s: bottom right point's longitude is too big: "
           "<%d>(max:%d): (%d,%d) (%d,%d)",
+          process_name,
           GRN_GEO_MAX_LONGITUDE, bottom_right->longitude,
           top_left->latitude, top_left->longitude,
           bottom_right->latitude, bottom_right->longitude);
       goto exit;
     }
+  }
+
+  {
+    int distance, latitude_distance, longitude_distance;
+    grn_geo_point *top_left, *bottom_right;
+    grn_geo_point *geo_point_input;
+    uint8_t geo_key_input[sizeof(grn_geo_point)];
+    uint8_t geo_key_base[sizeof(grn_geo_point)];
+
+    top_left = data->top_left;
+    bottom_right = data->bottom_right;
 
     latitude_distance = top_left->latitude - bottom_right->latitude;
     longitude_distance = bottom_right->longitude - top_left->longitude;
     if (latitude_distance > longitude_distance) {
-      direction = MESH_LATITUDE;
+      data->direction = GRN_GEO_MESH_LATITUDE;
       geo_point_input = bottom_right;
-      geo_point_base.latitude = bottom_right->latitude;
-      geo_point_base.longitude = bottom_right->longitude - longitude_distance;
+      data->base.latitude = bottom_right->latitude;
+      data->base.longitude = bottom_right->longitude - longitude_distance;
     } else {
-      direction = MESH_LONGITUDE;
+      data->direction = GRN_GEO_MESH_LONGITUDE;
       geo_point_input = top_left;
-      geo_point_base.latitude = top_left->latitude - latitude_distance;
-      geo_point_base.longitude = top_left->longitude;
+      data->base.latitude = top_left->latitude - latitude_distance;
+      data->base.longitude = top_left->longitude;
     }
     grn_gton(geo_key_input, geo_point_input, sizeof(grn_geo_point));
-    grn_gton(geo_key_base, &geo_point_base, sizeof(grn_geo_point));
-    diff_bit = compute_diff_bit(geo_key_input, geo_key_base);
-    compute_min_and_max(&geo_point_base, diff_bit,
-                        &geo_point_min, &geo_point_max);
-    if (direction == MESH_LATITUDE) {
-      distance = geo_point_max.latitude - geo_point_min.latitude + 1;
-      start = geo_point_min.latitude;
-      end = top_left->latitude;
-    } else {
-      distance = geo_point_max.longitude - geo_point_min.longitude + 1;
-      start = geo_point_min.longitude;
-      end = bottom_right->longitude;
+    grn_gton(geo_key_base, &(data->base), sizeof(grn_geo_point));
+    data->diff_bit = compute_diff_bit(geo_key_input, geo_key_base);
+    compute_min_and_max(&(data->base), data->diff_bit,
+                        &(data->min), &(data->max));
+    switch (data->direction) {
+    case GRN_GEO_MESH_LATITUDE :
+      data->distance = data->max.latitude - data->min.latitude + 1;
+      data->start = data->min.latitude;
+      data->end = top_left->latitude;
+      break;
+    case GRN_GEO_MESH_LONGITUDE :
+      data->distance = data->max.longitude - data->min.longitude + 1;
+      data->start = data->min.longitude;
+      data->end = bottom_right->longitude;
+      break;
     }
 #ifdef GEO_DEBUG
     printf("direction: %s\n",
-           direction == MESH_LATITUDE ? "latitude" : "longitude");
+           data->direction == GRN_GEO_MESH_LATITUDE ? "latitude" : "longitude");
     printf("base:         ");
-    grn_p_geo_point(ctx, &geo_point_base);
+    grn_p_geo_point(ctx, &(data->base));
     printf("input:        ");
     grn_p_geo_point(ctx, geo_point_input);
     printf("min:          ");
-    grn_p_geo_point(ctx, &geo_point_min);
+    grn_p_geo_point(ctx, &(data->min));
     printf("max:          ");
-    grn_p_geo_point(ctx, &geo_point_max);
+    grn_p_geo_point(ctx, &(data->max));
     printf("top-left:     ");
     grn_p_geo_point(ctx, top_left);
     printf("bottom-right: ");
     grn_p_geo_point(ctx, bottom_right);
-    printf("diff-bit:            %10d\n", diff_bit);
-    printf("start:               %10d\n", start);
-    printf("end:                 %10d\n", end);
-    printf("distance:            %10d\n", distance);
+    printf("diff-bit:            %10d\n", data->diff_bit);
+    printf("start:               %10d\n", data->start);
+    printf("end:                 %10d\n", data->end);
+    printf("distance:            %10d\n", data->distance);
     printf("distance(latitude):  %10d\n", latitude_distance);
     printf("distance(longitude): %10d\n", longitude_distance);
 #endif
-    memcpy(&geo_point_base, &geo_point_min, sizeof(grn_geo_point));
+    memcpy(&(data->base), &(data->min), sizeof(grn_geo_point));
+  }
 
-    for (i = start; i < end + distance; i += distance) {
-      grn_table_cursor *tc;
-      tc = grn_table_cursor_open(ctx, pat,
-                                 &geo_point_base, diff_bit,
-                                 NULL, 0,
-                                 0, -1,
-                                 GRN_CURSOR_PREFIX|GRN_CURSOR_SIZE_BY_BIT);
+exit :
+  return ctx->rc;
+}
+
+grn_obj *
+grn_geo_cursor_open_in_rectangle(grn_ctx *ctx,
+                                 grn_obj *index,
+                                 grn_obj *top_left_point,
+                                 grn_obj *bottom_right_point,
+                                 int offset,
+                                 int limit)
+{
+  grn_geo_cursor_in_rectangle *cursor = NULL;
+  in_rectangle_data data;
+
+  GRN_VOID_INIT(&(data.top_left_point_buffer));
+  GRN_VOID_INIT(&(data.bottom_right_point_buffer));
+  if (in_rectangle_data_prepare(ctx, index, top_left_point, bottom_right_point,
+                                "geo_in_rectangle()", &data)) {
+    goto exit;
+  }
+
+  cursor = GRN_MALLOCN(grn_geo_cursor_in_rectangle, 1);
+  if (!cursor) {
+    ERR(GRN_NO_MEMORY_AVAILABLE,
+        "[geo][cursor][in-rectangle] failed to allocate memory for geo cursor");
+    goto exit;
+  }
+
+  GRN_DB_OBJ_SET_TYPE(cursor, GRN_CURSOR_COLUMN_GEO_INDEX);
+  cursor->pat = data.pat;
+  cursor->index = index;
+  cursor->diff_bit = data.diff_bit;
+  cursor->start_mesh_point = data.start;
+  cursor->end_mesh_point = data.end;
+  cursor->distance = data.distance;
+  cursor->direction = data.direction;
+  memcpy(&(cursor->top_left), data.top_left, sizeof(grn_geo_point));
+  memcpy(&(cursor->bottom_right), data.bottom_right, sizeof(grn_geo_point));
+  memcpy(&(cursor->base), &(data.base), sizeof(grn_geo_point));
+  cursor->pat_cursor = NULL;
+  cursor->ii_cursor = NULL;
+  cursor->offset = offset;
+  cursor->rest = limit;
+
+exit :
+  grn_obj_unlink(ctx, &(data.top_left_point_buffer));
+  grn_obj_unlink(ctx, &(data.bottom_right_point_buffer));
+  return (grn_obj *)cursor;
+}
+
+typedef grn_bool (*grn_geo_cursor_callback)(grn_ctx *ctx, grn_ii_posting *posting, void *user_data);
+
+static void
+grn_geo_cursor_each(grn_ctx *ctx, grn_obj *geo_cursor,
+                    grn_geo_cursor_callback callback, void *user_data)
+{
+  grn_geo_cursor_in_rectangle *cursor;
+  grn_obj *pat;
+  grn_table_cursor *pat_cursor;
+  grn_ii *ii;
+  grn_ii_cursor *ii_cursor;
+  grn_ii_posting *posting = NULL;
+  grn_geo_point *current, *base, *top_left, *bottom_right;
+  int diff_bit, distance, end_mesh_point;
+  grn_geo_mesh_direction direction;
+  int i = 0;
+  int mesh_point;
+  grn_id index_id;
+
+  cursor = (grn_geo_cursor_in_rectangle *)geo_cursor;
+  if (cursor->rest == 0) {
+    return;
+  }
+
+  pat = cursor->pat;
+  pat_cursor = cursor->pat_cursor;
+  ii = (grn_ii *)(cursor->index);
+  ii_cursor = cursor->ii_cursor;
+  current = &(cursor->current);
+  base = &(cursor->base);
+  top_left = &(cursor->top_left);
+  bottom_right = &(cursor->bottom_right);
+  diff_bit = cursor->diff_bit;
+  distance = cursor->distance;
+  end_mesh_point = cursor->end_mesh_point;
+  direction = cursor->direction;
+
+  while (GRN_TRUE) {
+    if (!pat_cursor) {
+      if (!(cursor->pat_cursor = pat_cursor =
+            grn_table_cursor_open(ctx,
+                                  pat,
+                                  base,
+                                  diff_bit,
+                                  NULL, 0,
+                                  0, -1,
+                                  GRN_CURSOR_PREFIX|GRN_CURSOR_SIZE_BY_BIT))) {
+        cursor->rest = 0;
+        return;
+      }
 #ifdef GEO_DEBUG
-      printf("i:                   %10d\n", i);
+      {
+        switch (direction) {
+        case GRN_GEO_MESH_LATITUDE :
+          mesh_point = base->latitude;
+          break;
+        case GRN_GEO_MESH_LONGITUDE :
+          mesh_point = base->longitude;
+          break;
+        }
+        printf("mesh-point:          %10d\n", mesh_point);
+        inspect_mesh(ctx, base, diff_bit,
+                     (mesh_point - cursor->start_mesh_point) /
+                     distance);
+      }
 #endif
-      inspect_mesh(ctx, &geo_point_base, diff_bit, (i - start) / distance);
-      if (tc) {
-        grn_id tid;
-        grn_geo_point point;
-        while ((tid = grn_table_cursor_next(ctx, tc))) {
-          grn_table_get_key(ctx, pat, tid, &point, sizeof(grn_geo_point));
-          if (!grn_geo_in_rectangle_raw(ctx, &point, top_left, bottom_right)) {
+    }
+
+    while (ii_cursor || (index_id = grn_table_cursor_next(ctx, pat_cursor))) {
+      if (!ii_cursor) {
+        grn_table_get_key(ctx, pat, index_id, current, sizeof(grn_geo_point));
+        if (grn_geo_in_rectangle_raw(ctx, current, top_left, bottom_right)) {
+          inspect_tid(ctx, index_id, current, 0);
+          if (!(cursor->ii_cursor = ii_cursor =
+                grn_ii_cursor_open(ctx,
+                                   ii,
+                                   index_id,
+                                   GRN_ID_NIL,
+                                   GRN_ID_MAX,
+                                   ii->n_elements,
+                                   0))) {
             continue;
           }
-          inspect_tid(ctx, tid, &point, 0);
-          grn_ii_at(ctx, (grn_ii *)index, tid, (grn_hash *)res, op);
+        } else {
+          continue;
         }
-        grn_table_cursor_close(ctx, tc);
       }
-      if (direction == MESH_LATITUDE) {
-        geo_point_base.latitude += distance;
-      } else {
-        geo_point_base.longitude += distance;
+
+      while ((posting = grn_ii_cursor_next(ctx, ii_cursor))) {
+        if (cursor->offset == 0) {
+          grn_bool keep_each;
+          keep_each = callback(ctx, posting, user_data);
+          if (cursor->rest > 0) {
+            if (--(cursor->rest) == 0) {
+              keep_each = GRN_FALSE;
+            }
+          }
+          if (!keep_each) {
+            return;
+          }
+        } else {
+          cursor->offset--;
+        }
       }
+      grn_ii_cursor_close(ctx, ii_cursor);
+      cursor->ii_cursor = ii_cursor = NULL;
+    }
+    grn_table_cursor_close(ctx, pat_cursor);
+    cursor->pat_cursor = pat_cursor = NULL;
+
+    switch (direction) {
+    case GRN_GEO_MESH_LATITUDE :
+      mesh_point = (base->latitude += distance);
+      break;
+    case GRN_GEO_MESH_LONGITUDE :
+      mesh_point = (base->longitude += distance);
+      break;
+    }
+    if (mesh_point > end_mesh_point + distance) {
+      cursor->rest = 0;
+      return;
     }
   }
-exit :
-  grn_ii_resolve_sel_and(ctx, (grn_hash *)res, op);
+}
+
+static grn_bool
+grn_geo_cursor_next_callback(grn_ctx *ctx, grn_ii_posting *posting,
+                             void *user_data)
+{
+  grn_ii_posting **return_posting = user_data;
+  *return_posting = posting;
+  return GRN_FALSE;
+}
+
+grn_posting *
+grn_geo_cursor_next(grn_ctx *ctx, grn_obj *geo_cursor)
+{
+  grn_ii_posting *posting = NULL;
+  grn_geo_cursor_each(ctx, geo_cursor, grn_geo_cursor_next_callback, &posting);
+  return (grn_posting *)posting;
+}
+
+grn_rc
+grn_geo_cursor_close(grn_ctx *ctx, grn_obj *geo_cursor)
+{
+  grn_geo_cursor_in_rectangle *cursor;
+
+  if (!geo_cursor) { return GRN_INVALID_ARGUMENT; }
+
+  cursor = (grn_geo_cursor_in_rectangle *)geo_cursor;
+  if (cursor->pat) { grn_obj_unlink(ctx, cursor->pat); }
+  if (cursor->index) { grn_obj_unlink(ctx, cursor->index); }
+  if (cursor->pat_cursor) { grn_table_cursor_close(ctx, cursor->pat_cursor); }
+  if (cursor->ii_cursor) { grn_ii_cursor_close(ctx, cursor->ii_cursor); }
+  GRN_FREE(cursor);
+
+  return GRN_SUCCESS;
+}
+
+typedef struct {
+  grn_hash *res;
+  grn_operator op;
+} grn_geo_select_in_rectangle_data;
+
+static grn_bool
+grn_geo_select_in_rectangle_callback(grn_ctx *ctx, grn_ii_posting *posting,
+                                     void *user_data)
+{
+  grn_geo_select_in_rectangle_data *data = user_data;
+  grn_ii_posting_add(ctx, posting, data->res, data->op);
+  return GRN_TRUE;
+}
+
+grn_rc
+grn_geo_select_in_rectangle(grn_ctx *ctx, grn_obj *index,
+                            grn_obj *top_left_point,
+                            grn_obj *bottom_right_point,
+                            grn_obj *res, grn_operator op)
+{
+  grn_obj *cursor;
+  grn_posting *posting;
+
+  cursor = grn_geo_cursor_open_in_rectangle(ctx, index,
+                                            top_left_point, bottom_right_point,
+                                            0, -1);
+  if (cursor) {
+    grn_geo_cursor_in_rectangle *geo_cursor = (grn_geo_cursor_in_rectangle *)cursor;
+    grn_geo_select_in_rectangle_data data;
+    data.res = (grn_hash *)res;
+    data.op = op;
+    grn_geo_cursor_each(ctx, cursor, grn_geo_select_in_rectangle_callback,
+                        &data);
+    grn_obj_unlink(ctx, cursor);
+    grn_ii_resolve_sel_and(ctx, (grn_hash *)res, op);
+  }
+
   return ctx->rc;
 }
 
+static grn_rc
+geo_point_get(grn_ctx *ctx, grn_obj *pat, int flags, grn_geo_point *geo_point)
+{
+  grn_rc rc = GRN_SUCCESS;
+  grn_id id;
+  grn_table_cursor *cursor = NULL;
+
+  cursor = grn_table_cursor_open(ctx, pat,
+                                 NULL, 0,
+                                 NULL, 0,
+                                 0, 1,
+                                 GRN_CURSOR_BY_KEY | flags);
+  if (!cursor) {
+    rc = ctx->rc;
+    goto exit;
+  }
+
+  id = grn_table_cursor_next(ctx, cursor);
+  if (id == GRN_ID_NIL) {
+    rc = GRN_END_OF_DATA;
+  } else {
+    void *key;
+    int key_size;
+    key_size = grn_table_cursor_get_key(ctx, cursor, &key);
+    memcpy(geo_point, key, key_size);
+  }
+
+exit:
+  if (cursor) {
+    grn_table_cursor_close(ctx, cursor);
+  }
+  return rc;
+}
+
+int
+grn_geo_estimate_in_rectangle(grn_ctx *ctx,
+                              grn_obj *index,
+                              grn_obj *top_left_point,
+                              grn_obj *bottom_right_point)
+{
+  int n = 0;
+  int total_records;
+  grn_rc rc;
+  in_rectangle_data data;
+
+  GRN_VOID_INIT(&(data.top_left_point_buffer));
+  GRN_VOID_INIT(&(data.bottom_right_point_buffer));
+  if (in_rectangle_data_prepare(ctx, index, top_left_point, bottom_right_point,
+                                "grn_geo_estimate_in_rectangle()", &data)) {
+    n = -1;
+    goto exit;
+  }
+
+  total_records = grn_table_size(ctx, data.pat);
+  if (total_records > 0) {
+    grn_geo_point min, max;
+    int select_latitude_distance, select_longitude_distance;
+    int total_latitude_distance, total_longitude_distance;
+    double select_ratio;
+
+    rc = geo_point_get(ctx, data.pat, GRN_CURSOR_ASCENDING, &min);
+    if (!rc) {
+      rc = geo_point_get(ctx, data.pat, GRN_CURSOR_DESCENDING, &max);
+    }
+    if (rc) {
+      if (rc == GRN_END_OF_DATA) {
+        n = total_records;
+        rc = GRN_SUCCESS;
+      } else {
+        n = -1;
+      }
+      goto exit;
+    }
+
+    select_latitude_distance = abs(data.max.latitude - data.min.latitude);
+    select_longitude_distance = abs(data.max.longitude - data.min.longitude);
+    total_latitude_distance = abs(max.latitude - min.latitude);
+    total_longitude_distance = abs(max.longitude - min.longitude);
+
+    select_ratio = 1.0;
+    if (select_latitude_distance < total_latitude_distance) {
+      select_ratio *= ((double)select_latitude_distance /
+                       (double)total_latitude_distance);
+    }
+    if (select_longitude_distance < total_longitude_distance) {
+      select_ratio *= ((double)select_longitude_distance /
+                       (double)total_longitude_distance);
+    }
+    n = (int)(ceil(total_records * select_ratio));
+  }
+
+exit :
+  grn_obj_unlink(ctx, &(data.top_left_point_buffer));
+  grn_obj_unlink(ctx, &(data.bottom_right_point_buffer));
+  return n;
+}
+
 grn_bool
 grn_geo_in_circle(grn_ctx *ctx, grn_obj *point, grn_obj *center,
                   grn_obj *radius_or_point)
