@@ -1845,7 +1845,7 @@ set_cursor_ascend(grn_ctx *ctx, grn_pat *pat, grn_pat_cursor *c,
     c2 = len < ch ? len : ch;
     if ((check += 2) < c2) {
       if (!(k = pat_node_get_key(ctx, pat, node))) { return GRN_FILE_CORRUPT; }
-      if ((r = bitcmp(key, k, check >> 1, (c2 - check) >> 1))) {
+      if ((r = bitcmp(key, k, check >> 1, ((c2 + 1) >> 1) - (check >> 1)))) {
         if (r < 0) {
           push(c, node->lr[1], ch);
           push(c, node->lr[0], ch);
@@ -1854,7 +1854,11 @@ set_cursor_ascend(grn_ctx *ctx, grn_pat *pat, grn_pat_cursor *c,
       }
     }
     check = ch;
-    if (len <= check) { break; }
+    if (len <= check) {
+      push(c, node->lr[1], ch);
+      push(c, node->lr[0], ch);
+      break;
+    }
     if (check & 1) {
       if (check + 1 < len) {
         id = node->lr[1];
@@ -1909,7 +1913,7 @@ set_cursor_descend(grn_ctx *ctx, grn_pat *pat, grn_pat_cursor *c,
     c2 = len < ch ? len : ch;
     if ((check += 2) < c2) {
       if (!(k = pat_node_get_key(ctx, pat, node))) { return GRN_FILE_CORRUPT; }
-      if ((r = bitcmp(key, k, check >> 1, (c2 - check) >> 1))) {
+      if ((r = bitcmp(key, k, check >> 1, ((c2 + 1) >> 1) - (check >> 1)))) {
         if (r >= 0) {
           push(c, node->lr[0], ch);
           push(c, node->lr[1], ch);
@@ -1918,11 +1922,7 @@ set_cursor_descend(grn_ctx *ctx, grn_pat *pat, grn_pat_cursor *c,
       }
     }
     check = ch;
-    if (len <= check) {
-      push(c, node->lr[0], check);
-      push(c, node->lr[1], check);
-      break;
-    }
+    if (len <= check) { break; }
     if (check & 1) {
       if (check + 1 < len) {
         push(c, node->lr[0], check);
@@ -2015,7 +2015,11 @@ grn_pat_cursor_open_by_id(grn_ctx *ctx, grn_pat *pat,
       }
     }
   } else {
-    c->curr_rec += dir * offset;
+    if ((dir * (c->tail - c->curr_rec)) < offset) {
+      c->curr_rec = c->tail;
+    } else {
+      c->curr_rec += dir * offset;
+    }
   }
   c->rest = (limit < 0) ? GRN_ID_MAX : limit;
 exit :
@@ -2197,8 +2201,8 @@ grn_pat_check(grn_ctx *ctx, grn_pat *pat)
 {
   char buf[8];
   struct grn_pat_header *h = pat->header;
-  GRN_OUTPUT_ARRAY_OPEN("RESULT", 8);
-  GRN_OUTPUT_MAP_OPEN("SUMMARY", 8);
+  GRN_OUTPUT_ARRAY_OPEN("RESULT", 1);
+  GRN_OUTPUT_MAP_OPEN("SUMMARY", 11);
   GRN_OUTPUT_CSTR("flags");
   grn_itoh(h->flags, buf, 8);
   GRN_OUTPUT_STR(buf, 8);
