@@ -1,5 +1,6 @@
 /* -*- c-basic-offset: 2 -*- */
-/* Copyright(C) 2010 Brazil
+/*
+  Copyright(C) 2010-2011 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -15,6 +16,10 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif /* HAVE_CONFIG_H */
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -22,10 +27,6 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif /* HAVE_CONFIG_H */
-
 #ifdef HAVE_SYS_WAIT_H
 #include <sys/wait.h>
 #endif /* HAVE_SYS_WAIT_H */
@@ -36,7 +37,6 @@
 #include <netinet/in.h>
 #endif /* HAVE_NETINET_IN_H */
 
-#define __USE_XOPEN
 #include "lib/str.h"
 #include "lib/com.h"
 #include "lib/db.h"
@@ -76,11 +76,11 @@ static int grntest_remote_mode = 0;
 static int grntest_localonly_mode = 0;
 static int grntest_owndb_mode = 0;
 static int grntest_onmemory_mode = 0;
-static int grntest_noftp_mode = 0;
+static grn_bool grntest_ftp_mode = GRN_FALSE;
 #define TMPFILE "_grntest.tmp"
 
 static grn_ctx grntest_server_context;
-static FILE *grntest_logfp;
+static FILE *grntest_log_file;
 
 #define OS_LINUX64   "LINUX64"
 #define OS_LINUX32   "LINUX32"
@@ -322,56 +322,16 @@ escape_command(grn_ctx *ctx, char *in, int ilen, grn_obj *escaped_command)
   GRN_TEXT_PUTC(ctx, escaped_command, '\0');
 }
 
-
-static int
-report_load_command(grn_ctx *ctx, char *ret, int task_id, long long int start_time,
-                    grn_obj *end_time)
-{
-  int i, len;
-  long long int start, end;
-  char rettmp[BUF_LEN];
-
-  if ((ret[0] == '[') && (ret[1] == '[')) {
-    i = 2;
-    len = 1;
-    while (ret[i] != ']') {
-      i++;
-      len++;
-      if (ret[i] == '\0') {
-        fprintf(stderr, "Error results:load\n");
-        error_exit_in_thread(3);
-      }
-    }
-    len++;
-    strncpy(rettmp, &ret[1], len);
-    rettmp[len] = '\0';
-  } else {
-    strcpy(rettmp, ret);
-  }
-
-  start = start_time - GRN_TIME_VALUE(&grntest_starttime);
-  end = GRN_TIME_VALUE(end_time) - GRN_TIME_VALUE(&grntest_starttime);
-  if (grntest_outtype == OUT_TSV) {
-    fprintf(grntest_logfp, "report\t%d\tload\t%" GRN_FMT_LLD "\t%" GRN_FMT_LLD "\t%s\n",
-            task_id,  start, end, rettmp);
-  } else {
-    fprintf(grntest_logfp, "[%d, \"load\", %" GRN_FMT_LLD ", %" GRN_FMT_LLD ", %s],\n",
-            task_id,  start, end, rettmp);
-  }
-  fflush(grntest_logfp);
-  return 0;
-}
-
 static int
 report_command(grn_ctx *ctx, char *command, char *ret, int task_id,
                grn_obj *start_time, grn_obj *end_time)
 {
   int i, len, clen;
   long long int start, end;
-  char rettmp[BUF_LEN];
-  grn_obj escaped_command;
+  grn_obj result, escaped_command;
 
-  if ((ret[0] == '[') && (ret[1] == '[')) {
+  GRN_TEXT_INIT(&result, 0);
+  if (strncmp(ret, "[[", 2) == 0) {
     i = 2;
     len = 1;
     while (ret[i] != ']') {
@@ -383,10 +343,9 @@ report_command(grn_ctx *ctx, char *command, char *ret, int task_id,
       }
     }
     len++;
-    strncpy(rettmp, &ret[1], len);
-    rettmp[len] = '\0';
+    grn_text_esc(ctx, &result, ret + 1, len);
   } else {
-    strcpy(rettmp, ret);
+    grn_text_esc(ctx, &result, ret, strlen(ret));
   }
 
   start = GRN_TIME_VALUE(start_time) - GRN_TIME_VALUE(&grntest_starttime);
@@ -395,14 +354,17 @@ report_command(grn_ctx *ctx, char *command, char *ret, int task_id,
   GRN_TEXT_INIT(&escaped_command, 0);
   escape_command(ctx, command, clen, &escaped_command);
   if (grntest_outtype == OUT_TSV) {
-    fprintf(grntest_logfp, "report\t%d\t%s\t%" GRN_FMT_LLD "\t%" GRN_FMT_LLD "\t%s\n",
-            task_id, GRN_TEXT_VALUE(&escaped_command), start, end, rettmp);
+    fprintf(grntest_log_file, "report\t%d\t%s\t%" GRN_FMT_LLD "\t%" GRN_FMT_LLD "\t%.*s\n",
+            task_id, GRN_TEXT_VALUE(&escaped_command), start, end,
+            GRN_TEXT_LEN(&result), GRN_TEXT_VALUE(&result));
   } else {
-    fprintf(grntest_logfp, "[%d, \"%s\", %" GRN_FMT_LLD ", %" GRN_FMT_LLD ", %s],\n",
-            task_id, GRN_TEXT_VALUE(&escaped_command), start, end, rettmp);
+    fprintf(grntest_log_file, "[%d, \"%s\", %" GRN_FMT_LLD ", %" GRN_FMT_LLD ", %.*s],\n",
+            task_id, GRN_TEXT_VALUE(&escaped_command), start, end,
+            GRN_TEXT_LEN(&result), GRN_TEXT_VALUE(&result));
   }
-  fflush(grntest_logfp);
+  fflush(grntest_log_file);
   GRN_OBJ_FIN(ctx, &escaped_command);
+  GRN_OBJ_FIN(ctx, &result);
   return 0;
 }
 
@@ -421,9 +383,9 @@ output_result_final(grn_ctx *ctx, int qnum)
   sec = self / (double)1000000;
   qps = (double)qnum / sec;
   if (grntest_outtype == OUT_TSV) {
-    fprintf(grntest_logfp, "total\t%" GRN_FMT_LLD "\t%f\t%d\n", latency, qps, qnum);
+    fprintf(grntest_log_file, "total\t%" GRN_FMT_LLD "\t%f\t%d\n", latency, qps, qnum);
   } else {
-    fprintf(grntest_logfp,
+    fprintf(grntest_log_file,
            "{\"total\": %" GRN_FMT_LLD ", \"qps\": %f, \"queries\": %d}]\n", latency, qps, qnum);
   }
   grn_obj_close(ctx, &end_time);
@@ -434,9 +396,9 @@ static int
 output_sysinfo(char *sysinfo)
 {
   if (grntest_outtype == OUT_TSV) {
-    fprintf(grntest_logfp, "%s", sysinfo);
+    fprintf(grntest_log_file, "%s", sysinfo);
   } else {
-    fprintf(grntest_logfp, "[%s\n", sysinfo);
+    fprintf(grntest_log_file, "[%s\n", sysinfo);
   }
   return 0;
 }
@@ -647,7 +609,11 @@ command_send_http(grn_ctx *ctx, char *command, int type, int task_id)
 
   GRN_TEXT_INIT(&buf, 0);
   GRN_TEXT_PUTS(ctx, &buf, "GET ");
-  command_line_to_uri_path(ctx, &buf, command);
+  if (strncmp(command, "/d/", 3) == 0) {
+    GRN_TEXT_PUTS(ctx, &buf, command);
+  } else {
+    command_line_to_uri_path(ctx, &buf, command);
+  }
 #ifdef DEBUG_HTTP
   fprintf(stderr, "command: <%s>\n", command);
   fprintf(stderr, "path:    <%.*s>\n",
@@ -790,11 +756,12 @@ do_load_command(grn_ctx *ctx, char *command, int type, int task_id,
   int res_len, flags, ret;
   grn_obj start_time, end_time;
 
+  GRN_TIME_INIT(&start_time, 0);
   if (*load_start == 0) {
-    GRN_TIME_INIT(&start_time, 0);
     GRN_TIME_NOW(ctx, &start_time);
     *load_start = GRN_TIME_VALUE(&start_time);
-    grn_obj_close(ctx, &start_time);
+  } else {
+    GRN_TIME_SET(ctx, &start_time, *load_start);
   }
 
   command_send(ctx, command, type, task_id);
@@ -824,7 +791,7 @@ do_load_command(grn_ctx *ctx, char *command, int type, int task_id,
           strncpy(tmpbuf, res, BUF_LEN - 2);
           tmpbuf[BUF_LEN -2] = '\0';
         }
-        report_load_command(ctx, tmpbuf, task_id, *load_start, &end_time);
+        report_command(ctx, "load", tmpbuf, task_id, &start_time, &end_time);
       }
       if (out_p(grntest_task[task_id].jobtype)) {
         fwrite(res, 1, res_len, grntest_job[grntest_task[task_id].job_id].outputlog);
@@ -866,6 +833,7 @@ do_load_command(grn_ctx *ctx, char *command, int type, int task_id,
       break;
     }
   } while ((flags & GRN_CTX_MORE));
+    grn_obj_close(ctx, &start_time);
 
   return ret;
 }
@@ -1128,19 +1096,19 @@ exit:
     GRN_TEXT_PUTS(ctx, log, tmpbuf);
     if (grntest_jobdone == grntest_jobnum) {
       if (grntest_outtype == OUT_TSV) {
-        fprintf(grntest_logfp, "%.*s",
+        fprintf(grntest_log_file, "%.*s",
                 (int)GRN_TEXT_LEN(log), GRN_TEXT_VALUE(log));
       } else {
         if (grntest_detail_on) {
-          fseek(grntest_logfp, -2, SEEK_CUR);
-          fprintf(grntest_logfp, "], \n");
+          fseek(grntest_log_file, -2, SEEK_CUR);
+          fprintf(grntest_log_file, "], \n");
         }
-        fprintf(grntest_logfp, "\"summary\" :[");
-        fprintf(grntest_logfp, "%.*s",
+        fprintf(grntest_log_file, "\"summary\" :[");
+        fprintf(grntest_log_file, "%.*s",
                 (int)GRN_TEXT_LEN(log), GRN_TEXT_VALUE(log));
-        fprintf(grntest_logfp, "]");
+        fprintf(grntest_log_file, "]");
       }
-      fflush(grntest_logfp);
+      fflush(grntest_log_file);
     }
   }
   grn_obj_close(&grntest_ctx[task_id], &end_time);
@@ -2021,9 +1989,9 @@ printf("%d:type =%d:file=%s:con=%d:ntimes=%d\n", i, grntest_job[i].jobtype,
       ;
     }
     else {
-      fprintf(grntest_logfp, "\"detail\": [\n");
+      fprintf(grntest_log_file, "\"detail\": [\n");
     }
-    fflush(grntest_logfp);
+    fflush(grntest_log_file);
   }
 
   thread_main(ctx, task_num);
@@ -2106,15 +2074,15 @@ do_script(grn_ctx *ctx, const char *sfile)
       GRN_TIME_INIT(&grntest_jobs_start, 0);
       GRN_TIME_NOW(ctx, &grntest_jobs_start);
       if (grntest_outtype == OUT_TSV) {
-        fprintf(grntest_logfp, "jobs-start\t%s\n", buf);
+        fprintf(grntest_log_file, "jobs-start\t%s\n", buf);
       } else {
-        fprintf(grntest_logfp, "{\"jobs\": \"%s\",\n", buf);
+        fprintf(grntest_log_file, "{\"jobs\": \"%s\",\n", buf);
       }
       qnum = do_jobs(ctx, job_num, line);
       if (grntest_outtype == OUT_TSV) {
-        fprintf(grntest_logfp, "jobs-end\t%s\n", buf);
+        fprintf(grntest_log_file, "jobs-end\t%s\n", buf);
       } else {
-        fprintf(grntest_logfp, "},\n");
+        fprintf(grntest_log_file, "},\n");
       }
       qnum_total = qnum_total + qnum;
 
@@ -2771,13 +2739,14 @@ usage(void)
          "  -i, --host <ip/hostname>:  server address to listen (default: %s)\n"
          "  --localonly:               omit server connection\n"
          "  --log-output-dir:          specify output dir (default: current)\n"
-         "  --noftp:                   omit ftp connection\n"
+         "  --ftp:                     connect to ftp server\n"
          "  --onmemory:                load all commands into memory\n"
          "  --output-type <tsv/json>:  specify output-type (default: json)\n"
          "  --owndb:                   open dbs for each ctx\n"
          "  -p, --port <port number>:  server port number (default: %d)\n"
          "  --groonga <groonga_path>:  groonga command path (default: %s)\n"
-         "  --protocol <gqtp|http>:    groonga server protocol (default: %s)\n",
+         "  --protocol <gqtp|http>:    groonga server protocol (default: %s)\n"
+         "  --log-path <path>:         specify log file path\n",
           DEFAULT_DEST, DEFAULT_PORT,
           groonga_path, groonga_protocol);
   exit(1);
@@ -2790,7 +2759,7 @@ enum {
 };
 
 #define MODE_MASK      0x007f
-#define MODE_NOFTP     0x0080
+#define MODE_FTP       0x0080
 #define MODE_LOCALONLY 0x0100
 #define MODE_OWNDB     0x0800
 #define MODE_ONMEMORY  0x1000
@@ -2950,7 +2919,8 @@ main(int argc, char **argv)
   int qnum, i, mode = 0;
   grn_ctx context;
   char sysinfo[BUF_LEN];
-  char log[BUF_LEN];
+  char log_path_buffer[BUF_LEN];
+  const char *log_path = NULL;
   const char *portstr=NULL, *hoststr=NULL, *dbname=NULL, *scrname=NULL, *outdir=NULL, *outtype=NULL;
   time_t sec;
 
@@ -2960,13 +2930,14 @@ main(int argc, char **argv)
     {'\0', "log-output-dir", NULL, 0, getopt_op_none},
     {'\0', "output-type", NULL, 0, getopt_op_none},
     {'\0', "dir", NULL, mode_list, getopt_op_update},
-    {'\0', "noftp", NULL, MODE_NOFTP, getopt_op_on},
+    {'\0', "ftp", NULL, MODE_FTP, getopt_op_on},
     {'h', "help", NULL, mode_usage, getopt_op_update},
     {'\0', "localonly", NULL, MODE_LOCALONLY, getopt_op_on},
     {'\0', "onmemory", NULL, MODE_ONMEMORY, getopt_op_on},
     {'\0', "owndb", NULL, MODE_OWNDB, getopt_op_on},
     {'\0', "groonga", NULL, 0, getopt_op_none},
     {'\0', "protocol", NULL, 0, getopt_op_none},
+    {'\0', "log-path", NULL, 0, getopt_op_none},
     {'\0', NULL, NULL, 0, 0}
   };
 
@@ -2976,6 +2947,7 @@ main(int argc, char **argv)
   opts[3].arg = &outtype;
   opts[10].arg = &groonga_path;
   opts[11].arg = &groonga_protocol;
+  opts[12].arg = &log_path;
 
   i = grn_str_getopt(argc, argv, opts, &mode);
   if (i < 0) {
@@ -3018,8 +2990,8 @@ main(int argc, char **argv)
     grntest_onmemory_mode= 1;
   }
 
-  if (mode & MODE_NOFTP) {
-    grntest_noftp_mode= 1;
+  if (mode & MODE_FTP) {
+    grntest_ftp_mode = GRN_TRUE;
   }
 
   if ((scrname == NULL) || (dbname == NULL)) {
@@ -3048,7 +3020,7 @@ main(int argc, char **argv)
   grn_db_create(&grntest_server_context, NULL, NULL);
   grn_set_default_encoding(GRN_ENC_UTF8);
 
-  if (!grntest_noftp_mode) {
+  if (grntest_ftp_mode) {
     sync_script(&context, scrname);
   }
   check_script(scrname);
@@ -3072,17 +3044,24 @@ main(int argc, char **argv)
   sec = (time_t)(GRN_TIME_VALUE(&grntest_starttime)/1000000);
   get_date(grntest_date, &sec);
 
-  if (outdir) {
-    sprintf(log, "%s/%s-%s-%" GRN_FMT_LLD "-%s.log", outdir, grntest_scriptname,
-            grntest_username, GRN_TIME_VALUE(&grntest_starttime), grn_get_version());
-  } else {
-    sprintf(log, "%s-%s-%" GRN_FMT_LLD "-%s.log", grntest_scriptname,
-            grntest_username, GRN_TIME_VALUE(&grntest_starttime), grn_get_version());
+  if (!log_path) {
+    if (outdir) {
+      sprintf(log_path_buffer,
+              "%s/%s-%s-%" GRN_FMT_LLD "-%s.log", outdir, grntest_scriptname,
+              grntest_username,
+              GRN_TIME_VALUE(&grntest_starttime), grn_get_version());
+    } else {
+      sprintf(log_path_buffer,
+              "%s-%s-%" GRN_FMT_LLD "-%s.log", grntest_scriptname,
+              grntest_username,
+              GRN_TIME_VALUE(&grntest_starttime), grn_get_version());
+    }
+    log_path = log_path_buffer;
   }
 
-  grntest_logfp = fopen(log, "w+b");
-  if (!grntest_logfp) {
-    fprintf(stderr, "Cannot open logfile:%s\n", log);
+  grntest_log_file = fopen(log_path, "w+b");
+  if (!grntest_log_file) {
+    fprintf(stderr, "Cannot open log file: <%s>\n", log_path);
     goto exit;
   }
 
@@ -3094,13 +3073,13 @@ main(int argc, char **argv)
 #endif /* WIN32 */
   qnum = do_script(&context, scrname);
   output_result_final(&context, qnum);
-  fclose(grntest_logfp);
+  fclose(grntest_log_file);
 
-  if (!grntest_noftp_mode) {
-    ftp_sub(FTPUSER, FTPPASSWD, FTPSERVER, log, 3,
+  if (grntest_ftp_mode) {
+    ftp_sub(FTPUSER, FTPPASSWD, FTPSERVER, log_path, 3,
             "report", NULL);
   }
-  fprintf(stderr, "grntest done. logfile=%s\n", log);
+  fprintf(stderr, "grntest done. logfile=%s\n", log_path);
 
 exit:
   shutdown_server();
