@@ -188,14 +188,20 @@ prompt(grn_ctx *ctx, grn_obj *buf)
     line = el_wgets(edit_line, &nchar);
     if (nchar > 0) {
       int i;
-      char multibyte_buf[10]; /* enough for a wide char? */
+      char multibyte_buf[MB_CUR_MAX];
       size_t multibyte_len;
       mbstate_t ps;
       history_w(command_history, &command_history_event, H_ENTER, line);
       wcrtomb(NULL, L'\0', &ps);
       for (i = 0; i < nchar; i++) {
         multibyte_len = wcrtomb(multibyte_buf, line[i], &ps);
-        GRN_TEXT_PUT(ctx, buf, multibyte_buf, multibyte_len);
+        if (multibyte_len == (size_t)-1) {
+          GRN_LOG(ctx, GRN_LOG_WARNING,
+                  "[prompt][libedit] failed to read input: %s", strerror(errno));
+          rc = GRN_INVALID_ARGUMENT;
+        } else {
+          GRN_TEXT_PUT(ctx, buf, multibyte_buf, multibyte_len);
+        }
       }
       rc = GRN_SUCCESS;
       len = GRN_TEXT_LEN(buf);
