@@ -1,6 +1,6 @@
 /* -*- c-basic-offset: 2; coding: utf-8 -*- */
 /*
-  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
+  Copyright (C) 2008-2011  Kouhei Sutou <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -34,6 +34,10 @@ void test_open_invalid_chunk_file(void);
 void test_open_with_null_lexicon(void);
 void test_crud(void);
 void test_array_index(void);
+void test_scalar_index(void);
+void test_int_index(void);
+void test_mroonga_index(void);
+void test_mroonga_index_score(void);
 
 #define TYPE_SIZE 1024
 
@@ -152,6 +156,7 @@ cut_teardown(void)
     inverted_index_free();
     if (path)
       grn_ii_remove(context, path);
+    grn_obj_close(context, db);
     grn_ctx_fin(context);
     g_free(context);
   }
@@ -458,6 +463,8 @@ test_array_index(void)
   gchar *db_path, *name;
   grn_obj *users, *items, *checks, *checked;
 
+  grn_obj_close(context, db);
+
   remove_tmp_directory();
   g_mkdir_with_parents(tmp_directory, 0700);
   db_path = g_build_filename(tmp_directory, "inverted-index", NULL);
@@ -500,6 +507,8 @@ test_scalar_index(void)
   gchar *db_path, *name;
   grn_obj *users, *items, *checks, *checked;
 
+  grn_obj_close(context, db);
+
   remove_tmp_directory();
   g_mkdir_with_parents(tmp_directory, 0700);
   db_path = g_build_filename(tmp_directory, "inverted-index", NULL);
@@ -542,6 +551,8 @@ test_int_index(void)
   gchar *db_path, *name;
   grn_obj *users, *items, *checks, *checked, *int_type;
 
+  grn_obj_close(context, db);
+
   remove_tmp_directory();
   g_mkdir_with_parents(tmp_directory, 0700);
   db_path = g_build_filename(tmp_directory, "inverted-index", NULL);
@@ -615,6 +626,7 @@ test_mroonga_index(void)
   mrn_dir = cut_build_path(tmp_directory, "mrn", NULL);
   g_mkdir_with_parents(mrn_dir, 0700);
 
+  grn_obj_close(context, db);
   db = grn_db_create(context,
                      cut_build_path(mrn_dir, "mroonga.grn", NULL),
                      NULL);
@@ -746,6 +758,7 @@ test_mroonga_index_score(void)
   mrn_dir = cut_build_path(tmp_directory, "mrn", NULL);
   g_mkdir_with_parents(mrn_dir, 0700);
 
+  grn_obj_close(context, db);
   db = grn_db_create(context,
                      cut_build_path(mrn_dir, "mroonga.grn", NULL),
                      NULL);
